import { IOffset, IOrigin, IPoint, Location, position } from '../../Utilities/Position';
export const enum ContentJustification {
    Start = 0,
    Center = 1,
    End = 2,
    Stretch = 3
}

export const enum ContentLocation {
    Start = 0,
    Center = 1,
    End = 2
}

export const enum ContentOrientation {
    Column = 0,
    Row = 1
}

export const enum ContentSize {
    Small = 0,
    Medium = 1,
    Large = 2
}

/**
 * ICalloutProps is used to define a component that is detached from the DOM tree
 * the component is created within. It uses a React.Portal to ensure that messages
 * still flow through the component hierarchy.
 *
 * The properties of the callout are designed to position the element in the page
 * since the element in the DOM is not relative to the element we want it to still
 * propagate events through the React Component tree.
 *
 * The callout has two distinct layout modes:
 *  1) Absolute or Element relative position - For this you used anchorElement or
 *      anchorPoint for the basis of the callout.
 *
 *  2) Window relative position - For this you use callout content properties, like
 *      contentJustification, contentLocation, contentOrientation, and contentSize.
 *
 *      You dont need to supply them all, for example if you want to it auto size
 *      just dont pass a contentSize and it will take on the size of the layout.
 */
export interface ICalloutProps {
    /**
     * If the callout is being positioned relative to an existing element in the DOM
     * the relative element should be supplied as the anchorElement.
     */
    anchorElement?: HTMLElement;

    /**
     * anchorOffset is used to shift the location of the callout from the defined
     * location by a fixed amount. An example might be the tooltip callout defaults
     * to an anchorOffset of { x: 8 y: 8 } from the mouse position.
     */
    anchorOffset?: IOffset;

    /**
     * When an anchorElement is supplied the anchorOriging is used to describe the
     * relative location of the callout with respect to the anchorElement. A common
     * example here is a dropdown button aligns the callout to the bottom-right of
     * the button.
     */
    anchorOrigin?: IOrigin;

    /**
     * Instead of using an anchorElement to position the callout, if can be positioned
     * based on a fixed location. The anchorPoint defines the fixed position for the
     * anchor.
     */
    anchorPoint?: IPoint;

    /**
     * ariaLabel allows the root element to describe the elements contents to assistive
     * technology.
     */
    ariaLabel?: string;

    /**
     * calloutOrigin works in conjunction with the other position styles;
     * anchorElement, anchorOffset, anchorOrigin, anchorPoint. After determining the
     * location based on the anchor properties the position of the callout is applied.
     * The means the callout positioned with the calloutOrigin matching the anchorPosition.
     *
     * Example: Again the DropDownButton uses anchorOrigin -> Bottom, right with no offset.
     * This means the anchorPoint will be the bottom right of the button. The calloutOrigin
     * is defined as top, right. This will position the top, right corner of the callout
     * on the bottom right of the button.
     */
    calloutOrigin?: IOrigin;

    /**
     * Optional css className to apply to the callout element.
     */
    className?: string;

    /**
     * Optional css className to apply to the content element of the callout.
     */
    contentClassName?: string;

    /**
     * Justification of the callout content within the callout. This defaults to auto
     * layout and will take on the content size.
     */
    contentJustification?: ContentJustification;

    /**
     * Window relative location of the callout. This needs to be supplied for a window
     * relative layout.
     */
    contentLocation?: ContentLocation;

    /**
     * Orientation of the callout, is this a column or a row.
     */
    contentOrientation?: ContentOrientation;

    /**
     * Should the callout have the standard callout shadow.
     */
    contentShadow?: boolean;

    /**
     * If supplied there are a set of well-known fixed size callout's designed to keep
     * things like panels and other types of information using a consistent layout. This
     * should be supplied when you are following a standard UI model.
     */
    contentSize?: ContentSize;

    /**
     * hostElement is used to control where in the DOM the callout is attached.
     * By default it will be added the body if not hostElement or hostSelector are
     * supplied.
     */
    hostElement?: HTMLElement;

    /**
     * hostSelector is used when no hostElement is supplied to determine the
     * location in the DOM the callout will be attached.
     */
    hostSelector?: string;

    /**
     * If the id supplied, this id will be added to the callout element as the DOM
     * id attribute.
     */
    id?: string;

    /**
     * The callout will handle keyboard events and when the escape key is pressed the
     * callout will close if the the event hasnt had the defaultPrevented.
     */
    isEscDismiss?: boolean;

    /**
     * The layer element will trap mouse events and dismiss the callout when a click
     * occurs. This will prevent the events from being handled by the underlying
     * elements.
     */
    isLightDismiss?: boolean;

    /**
     * The layer element will be given a class that is a semi-transparency that helps
     * the user understand the user should focus on the callout and dismiss it before
     * interacting with the underlying of the document.
     */
    isModal?: boolean;

    /**
     * Method that is called when the callout is dismissed.
     */
    onDismiss?: () => void;

    /**
     * role isused to define the assistive usage of this callout. The default is
     * "dialog", but can be overriden with this property.
     */
    role?: string;
}