import * as tslib_1 from "tslib";
import "../../Core/core.scss";
import "../../Common.scss";
import * as React from "react";
import { ObservableLike } from '../../Core/Observable';
/**
 * Handles subscription to properties that are IObservableValues, so that components don't have to handle on their own.
 *
 * Usage:
 *
 * <Observer myObservableValue={observableValue} className='foo'>
 *     <MyComponent myObservableValue='' />
 * </Observer>
 *
 * Your component will get re-rendered with the new value of myObservableValue whenever that value changes.
 * Additionally, any additional props set on the Observer will also get passed down.
 */
var Observer = /** @class */ (function (_super) {
    tslib_1.__extends(Observer, _super);
    function Observer(props) {
        var _this = _super.call(this, props) || this;
        _this.subscriptions = {};
        // Initialize the state with the initial value of the observable.
        var state = { values: {}, oldProps: {} };
        for (var propName in props) {
            state.values[propName] = ObservableLike.getValue(props[propName]);
        }
        _this.state = state;
        return _this;
    }
    Observer.getDerivedStateFromProps = function (props, state) {
        var newState = Observer.updateSubscriptionsAndState(state.oldProps, props, state);
        if (newState != null) {
            return tslib_1.__assign({}, newState, { oldProps: props });
        }
        return tslib_1.__assign({}, newState, { oldProps: props });
    };
    Observer.prototype.render = function () {
        var newProps = {};
        // Copy over any properties from the observable component to the children.
        for (var key in this.state.values) {
            if (key !== "children") {
                newProps[key] = this.state.values[key];
            }
        }
        if (typeof this.props.children === "function") {
            var child = this.props.children;
            return child(newProps);
        }
        else {
            var child = React.Children.only(this.props.children);
            return React.cloneElement(child, Object.assign({}, child.props, newProps), child.props.children);
        }
    };
    Observer.prototype.componentDidMount = function () {
        this.updateSubscriptionsAndStateAfterRender();
    };
    Observer.prototype.componentDidUpdate = function () {
        this.updateSubscriptionsAndStateAfterRender();
    };
    Observer.prototype.componentWillUnmount = function () {
        // Unsubscribe from any of the observable proeprties.
        for (var propName in this.subscribedProps) {
            this.unsubscribe(propName, this.subscribedProps);
        }
    };
    Observer.prototype.updateSubscriptionsAndStateAfterRender = function () {
        var newState = Observer.updateSubscriptionsAndState(this.subscribedProps, this.props, this.state, this);
        if (newState != null) {
            this.setState(newState);
        }
        this.subscribedProps = tslib_1.__assign({}, this.props);
    };
    Observer.updateSubscriptionsAndState = function (oldProps, newProps, state, component) {
        // We need to unsubscribe from any observable values on old props and
        // subscribe to any observable values on new props.
        // In addition, if any of the values of the observables on the new props
        // differ from the value on the state, then we need to update the state.
        // This is possible if the value of the observable changed while the value
        // was being rendered, but before we had set up the subscription.
        // If we want to unsubscribe/resubscribe, then a component should be passed,
        // since this method is always called statically.
        var newState = tslib_1.__assign({}, state);
        var stateChanged = false;
        if (oldProps) {
            for (var propName in oldProps) {
                var oldValue = oldProps[propName];
                var newValue = newProps[propName];
                if (oldValue !== newValue) {
                    component && component.unsubscribe(propName, oldProps);
                    if (newValue === undefined) {
                        delete newState.values[propName];
                        stateChanged = true;
                    }
                }
            }
        }
        for (var propName in newProps) {
            var oldValue = oldProps && oldProps[propName];
            var newValue = newProps[propName];
            if (oldValue !== newValue) {
                component && component.subscribe(propName, newProps);
                // Look for changes in the observables between creation and now.
                if (state.values[propName] !== ObservableLike.getValue(newValue)) {
                    newState.values[propName] = ObservableLike.getValue(newValue);
                    stateChanged = true;
                }
            }
        }
        // If any state updates occurred update the state now.
        if (stateChanged) {
            return newState;
        }
        return null;
    };
    Observer.prototype.subscribe = function (propName, props) {
        var observableValue = props[propName];
        if (ObservableLike.isObservable(observableValue)) {
            var delegate = this.onValueChanged.bind(this, propName, observableValue);
            ObservableLike.subscribe(observableValue, delegate);
            this.subscriptions[propName] = delegate;
        }
    };
    Observer.prototype.unsubscribe = function (propName, props) {
        var observableValue = props[propName];
        if (ObservableLike.isObservable(observableValue)) {
            var delegate = this.subscriptions[propName];
            ObservableLike.unsubscribe(observableValue, delegate);
            delete this.subscriptions[propName];
        }
    };
    Observer.prototype.onValueChanged = function (propName, observableValue, value, action) {
        var _a;
        this.setState(tslib_1.__assign({}, this.state, { values: tslib_1.__assign({}, this.state.values, (_a = {}, _a[propName] = observableValue.value || value, _a)) }));
    };
    return Observer;
}(React.Component));
export { Observer };
