import * as React from "react";
import { IBehavior } from '../../Utilities/Behavior';
import { IEventDispatch } from '../../Utilities/Dispatch';
import { IFocusZoneProps } from '../../FocusZone';
import { IList, IListRow } from '../../List';
import { IReadonlyObservableValue } from '../../Core/Observable';
import { IItemProvider } from '../../Utilities/Provider';
import { IListSelection } from '../../Utilities/Selection';
import { IIconProps } from '../../Icon';
export interface ITable<T> extends IList<T> {
    cellFromPoint(clientX: number, clientY: number): ICellPosition;
}

/**
 *
 */
export interface ITableProps<T> extends ITableRowRenderer<T> {
    columns: Array<ITableColumn<T>>;

    behaviors?: Array<IBehavior<Partial<ITableProps<T>>, Partial<ITable<T>>>>;
    className?: string;
    eventDispatch?: IEventDispatch;
    focuszoneProps?: IFocusZoneProps;
    hideHeader?: boolean;
    hideLines?: boolean;
    id?: string;
    itemProvider: IItemProvider<T>;
    pageSize?: number;
    role?: string;
    rowHeight?: number;
    scrollable?: boolean;
    selection?: IListSelection;

    onRenderHeader?: (columns: Array<ITableColumn<T>>, table: ITable<T>) => JSX.Element;

    onActivate?: (event: React.SyntheticEvent<HTMLElement>, listRow: IListRow<T>) => void;
    onSelect?: (event: React.SyntheticEvent<HTMLElement>, listRow: IListRow<T>) => void;
}

/**
 * The ColumnStyles effect how the values for the column should be renderd.
 *
 * Primary columns should be rendered with emphasis.
 * Secondary colums should be rendered normally.
 * Tertiary columns should be rendered normally and are responsive by default.
 *  Being responsive means when the horizontal space is limited it will remove the column.
 */
export const enum TableColumnStyle {
    Secondary = 0,
    Primary = 1,
    Tertiary = 2
}

/**
 * ITableColumn is used to commnuicate details about the column, its layout
 * styles, widths, and basic header information. It requires a function to
 * render the contents of the cell be supplied.
 */
export interface ITableColumn<T> {
    id: string;
    width: IReadonlyObservableValue<number> | number;

    ariaLabel?: string;
    className?: string;
    columnStyle?: TableColumnStyle;
    disabled?: boolean;
    headerClassName?: string;
    iconProps?: IIconProps;
    maxWidth?: number;
    minWidth?: number;
    name?: string;

    onRenderCell: (rowIndex: number, columnIndex: number, listColumn: ITableColumn<T>, listItem: T) => JSX.Element;

    initialize?: (props: ITableProps<T>, table: ITable<T>, eventDispatch: IEventDispatch) => void;
    onRenderHeaderCell?: (index: number, tableColumn: ITableColumn<T>, focuszoneId: string | undefined) => JSX.Element;
    onSize?: (index: number, width: number) => void;
}

export interface ITableHeaderCellProps<T> {
    column: ITableColumn<T>;
    columnIndex: number;
    focuszoneId?: string;
}

export interface ITableColumnGroupProps<T> {
    columns: Array<ITableColumn<T>>;
}

/**
 * If an item being renfered in the table wants to control rendering of the row
 * for this specific item it can implement the ITableRowRenderer interface
 * and supply the onRenderRow function. This will hand over rendering of the
 * entire row the item instead of using the default renderer supplied to the
 * details list props.
 */
export interface ITableRowRenderer<T> {
    onRenderRow?: (
        index: number,
        item: T,
        position: number,
        setSize: number,
        columns: Array<ITableColumn<T>>,
        onFocusItem: (index: number, event: React.FocusEvent<HTMLElement>) => void,
        tableProps: ITableProps<T>,
        eventDispatch: IEventDispatch
    ) => JSX.Element;
}

export interface ITableRowProps<T> {
    columns: Array<ITableColumn<T>>;
    tableProps: ITableProps<T>;
    eventDispatch: IEventDispatch;
    index: number;
    item: T;
    position: number;
    setSize: number;

    onFocusItem: (index: number, event: React.FocusEvent<HTMLElement>) => void;
}

export interface ITableCell {
    [prop: string]: IListCell | string | number;
}

export interface ICellPosition {
    columnIndex: number;
    rowIndex: number;
}