import { format } from '../Core/Util/String';
import * as PlatformResources from '../Resources.Core';
export var millisecond = 1;
export var second = millisecond * 1000;
export var minute = second * 60;
export var hour = minute * 60;
export var day = hour * 24;
export var week = day * 7;
;
/**
 * Generate a string indicating how long ago the date is.
 *
 * @param date The Date object to format
 * @param agoFormat The required format
 * @param now The Dateobject of reference time
 * @param locale The current locale
 * @return A friendly string
 */
export function ago(date, agoFormat, now, locale) {
    if (!now) {
        now = new Date();
    }
    if (!agoFormat) {
        agoFormat = 0 /* Compact */;
    }
    // Getting the difference in milliseconds between now and the specified date
    var diff = now.getTime() - date.getTime();
    var result = "";
    if (diff < 2 * minute) {
        result = PlatformResources.JustNow;
    }
    else if (agoFormat === 0 /* Compact */) {
        if (diff < 59 * minute) {
            result = format(PlatformResources.AgoMinutes, Math.round(diff / minute));
        }
        else if (now.toDateString() === date.toDateString()) {
            result = format(PlatformResources.AgoHours, Math.round(diff / hour));
        }
        else if (diff < week && now.getDay() !== date.getDay()) {
            var yesterday = new Date();
            yesterday.setDate(now.getDate() - 1);
            if (date.getDate() === yesterday.getDate()) {
                result = PlatformResources.Yesterday;
            }
            else {
                var options = { weekday: 'long' };
                result = date.toLocaleString(locale, options);
            }
        }
        else if (now.getFullYear() === date.getFullYear()) {
            var options = { month: 'short', day: 'numeric' };
            result = date.toLocaleDateString(locale, options);
        }
        else {
            var options = { month: 'short', year: 'numeric', day: 'numeric' };
            result = date.toLocaleDateString(locale, options);
        }
    }
    else if (agoFormat === 1 /* Extended */) {
        var dayPart = "";
        if (now.toDateString() === date.toDateString()) {
            dayPart = PlatformResources.Today;
        }
        else if (diff < week && now.getDay() !== date.getDay()) {
            var yesterday = new Date();
            yesterday.setDate(now.getDate() - 1);
            if (date.getDate() === yesterday.getDate()) {
                dayPart = PlatformResources.Yesterday;
            }
            else {
                var options = { weekday: 'short' };
                dayPart = date.toLocaleString(locale, options);
            }
        }
        else if (now.getFullYear() === date.getFullYear()) {
            var options = { month: 'short', day: 'numeric' };
            dayPart = date.toLocaleDateString(locale, options);
        }
        else {
            var options = { month: 'short', year: 'numeric', day: 'numeric' };
            dayPart = date.toLocaleDateString(locale, options);
        }
        var timePart = date.toLocaleString(locale, { hour: 'numeric', minute: 'numeric' });
        result = format(PlatformResources.DayAtTimeFormat, dayPart, timePart);
    }
    if (!result) {
        result = date.toString();
    }
    return result;
}
;
/**
 * Returns string representation for date to be used in log.
 *
 * @param date The Date object to format
 * @param logType The required format
 * @param locale The current locale
 * @return A string representation of date in requested format
 */
export function log(date, logType, locale) {
    var result = "";
    if (!logType) {
        logType = 0 /* Full */;
    }
    if (0 /* Full */ === logType) {
        result = date.toISOString();
    }
    else if (1 /* Date */ === logType) {
        var dayOptions = { month: "2-digit", day: "2-digit" };
        var dayPart = date.toLocaleDateString(locale, dayOptions);
        var timeOptions = { hour: "2-digit", minute: "2-digit", hour12: false, timeZoneName: "short" };
        var timePart = date.toLocaleTimeString(locale, timeOptions);
        result = dayPart + " " + timePart;
    }
    else if (2 /* Local */ === logType) {
        var timeOptions = { hour: "2-digit", minute: "2-digit", second: "2-digit", hour12: false, timeZoneName: "short" };
        result = date.toLocaleTimeString(locale, timeOptions);
    }
    if (!result) {
        result = date.toString();
    }
    return result;
}
/**
 * Returns string representation for date to be used in duration.
 *
 * @param startDate The Date object to format
 * @param endDate The Dateobject of reference time
 * @return A string representation of date in requested format
 */
export function duration(startDate, endDate) {
    if (!endDate) {
        endDate = new Date();
    }
    // Getting the difference in seconds between now and the specified date
    var diff = endDate.getTime() - startDate.getTime();
    var result = "";
    if (diff < second) {
        result = PlatformResources.LessThanASecond;
    }
    else if (diff < minute) {
        result = format(PlatformResources.Seconds, Math.floor(diff / second));
    }
    else {
        var days = Math.floor(diff / day);
        var hours = Math.floor((diff % day) / hour);
        var minutes = Math.floor((diff % hour) / minute);
        var seconds = Math.floor((diff % minute) / second);
        if (diff < hour) {
            result = format(PlatformResources.MinutesSeconds, minutes, seconds);
        }
        else if (diff < day) {
            result = format(PlatformResources.HoursMinutesSeconds, hours, minutes, seconds);
        }
        else {
            result = format(PlatformResources.DaysHoursMinutes, days, hours, minutes);
        }
    }
    return result;
}
/**
 * Returns string representation for date to be used as a tooltip for ago and duration.
 *
 * @param date The Date object to format
 * @param locale The current locale
 * @return A string representation of date in requested format
 */
export function tooltipString(date, locale) {
    var dayOptions = { month: 'short', year: 'numeric', day: 'numeric' };
    var dayPart = date.toLocaleDateString(locale, dayOptions);
    var timeOptions = { hour: 'numeric', minute: 'numeric', timeZoneName: 'short' };
    var timePart = date.toLocaleTimeString(locale, timeOptions);
    var result = format(PlatformResources.DayAtTimeFormat, dayPart, timePart);
    return result;
}
