/**
 * The position method is used to set the location of an absolutely positioned element
 * using the standard positioning properties. The names of these properties conform to
 * the naming patterns used in the Material Popover https://material-ui.com. They
 * are not exact but follow the same pattern.
 *
 * For an example usage, look at the Callout component and how it uses this method to
 * position the element in the page.
 *
 * @param transformElement The element that is being positioned/transformed.
 * @param transformOrigin The origin within the transformed element to align with the
 *  anchor position.
 * @param anchorOffset Offset on the anchorElement that is applied to the computed location
 *  given the element/origin/point.
 * @param anchorElement The element used to anchor the position of the transformed element.
 *  The caller must supply either an anchorElement and anchorOrigin, or anchorPoint.
 * @param anchorOrigin When an anchorElement is supplie the anchorOrigin defines the location
 *  on the anchorElement used for positioning.
 * @param anchorPoint Instead of an anchorElement the caller can use an explicit point
 *  to be used as the basis for the anchorLocation. The anchorOffset will still be applied.
 * @param pageOffset If the caller is changing the DOM and doesnt want a force reflow,
 *  they may pass the current pageOffset to be used. Note that if the page changes/scrolls
 *  while the element is being laid out the positioning may not be accurate.
 */
export function position(transformElement, transformOrigin, anchorOffset, anchorElement, anchorOrigin, anchorPoint, pageOffset) {
    // Translate the anchor location information to a point on the anchor element
    // if a specific point was not supplied.
    if (!anchorPoint) {
        if (anchorElement && anchorOrigin) {
            anchorPoint = pointFromOrigin(anchorOrigin, anchorElement.getBoundingClientRect());
        }
        else {
            return;
        }
    }
    var left = anchorPoint.x;
    var top = anchorPoint.y;
    // Update the anchorPoint by the anchorOffset if one was supplied.
    if (anchorOffset) {
        left += anchorOffset.horizontal;
        top += anchorOffset.vertical;
    }
    // Special case Start/Start since we dont need the transformElement rect.
    if (transformOrigin.horizontal !== "start" /* start */ || transformOrigin.vertical !== "start" /* start */) {
        var rectTransform = transformElement.getBoundingClientRect();
        // Compute the effective horizontal position of the element.
        switch (transformOrigin.horizontal) {
            case "end" /* end */:
                left -= rectTransform.width;
                break;
            case "end" /* end */:
                left -= rectTransform.width / 2;
                break;
            default:
        }
        // Compute the effective vertical position of the element.
        switch (transformOrigin.vertical) {
            case "end" /* end */:
                top -= rectTransform.height;
                break;
            case "end" /* end */:
                top -= rectTransform.height / 2;
                break;
            default:
        }
    }
    // Adjust the position for screen relative position of the anchor.
    left += pageOffset ? pageOffset.x : window.pageXOffset;
    top += pageOffset ? pageOffset.y : window.pageYOffset;
    // If an offset was supplied we will apply it to the
    // Update the transform elements position (it needs to be absolutely positioned in the window).
    transformElement.style.left = left + "px";
    transformElement.style.top = top + "px";
}
function pointFromOrigin(location, rect) {
    var x;
    var y;
    // Compute the horizontal position based on the rectangle.
    switch (location.horizontal) {
        case "start" /* start */:
            x = rect.left;
            break;
        case "end" /* end */:
            x = rect.right;
            break;
        default:
            x = rect.left + rect.width / 2;
    }
    // Compute the vertical position based on the rectangle.
    switch (location.vertical) {
        case "start" /* start */:
            y = rect.top;
            break;
        case "end" /* end */:
            y = rect.bottom;
            break;
        default:
            y = rect.top + rect.height / 2;
    }
    return { x: x, y: y };
}
