/**
 * This utility is used to post-process CSS containing CSS variables so that a default (non-themed) version works
 * on browsers that don't support CSS variables (IE 11).
 * 
 * This requires a JSON file containing the default values for all possible CSS variables. It will
 * find any var(--XXX) values in CSS and emit 2 styles, one with the default value, then a second
 * (higher precedence) that uses the CSS variable.
 * 
 * So the following:
 * 
 *   .button {
 *       color: var(--button-color);
 *   }
 * 
 * becomes:
 * 
 *   .button {
 *       color: #ccc;
 *       color: var(--button-color, #ccc);
 *   }
 * 
 * given a defaults file like:
 * 
 *   {
 *       "button-color": "#ccc"
 *   }
 */

const fs = require("fs");
const path = require("path");

const varString = "var(--";
const borderAttributes = {
    "border": true,
    "border-top": true,
    "border-right": true,
    "border-bottom": true,
    "border-left": true
};

function processCssLine(line, defaults, outputLines) {

    let writeOriginalLine = true;

    // Look to see if line might have an attribute
    let indexOfAttribute = line.indexOf(":");

    // If there is an attribute, then look for a var function.  var function will start witht  var(--
    if (indexOfAttribute > -1) {

        const indexOfVar = line.indexOf(varString, indexOfAttribute);
        if (indexOfVar > -1) {

            // There is a var function so we will be writing a duplicate attribute.
            const attribute = line.substr(0, indexOfAttribute);

            // Special-case handling for the "border" attribute for a bug in Safari where it doesn't support
            // rgba with a variable unless it is at the start of the border attribute. We will pull this out
            // into two separate statements. A border: with the width and fill, and a border-color with the
            // rgba/variable combo.
            //
            // Safari bug: https://bugs.webkit.org/show_bug.cgi?id=185940
            //
            // Example:
            //     border: 1px solid rgba(var(--palette-color), 0.1);
            //
            // becomes:
            //     border: 1px solid;
            //     border-color: rgba(var(--palette-color), 0.1);
            //
            // then gets each of these lines re-processed

            let rgbaIndex;
            if (borderAttributes[attribute.trim()] && (rgbaIndex = line.indexOf(" rgba(", indexOfAttribute + 1)) !== -1) {
                let preRgba = line.substr(indexOfAttribute + 1, rgbaIndex - indexOfAttribute - 1);
                if (preRgba) {
                    processCssLine(`${attribute}: ${preRgba};`, defaults, outputLines);
                    processCssLine(`${attribute}-color: ${line.substr(rgbaIndex + 1)}`, defaults, outputLines);
                    return;
                }
            }

            // Store everything from after attribute and before the var
            const prevar = line.substr(indexOfAttribute + 1, indexOfVar - indexOfAttribute - 1);

            // Need to find the end of the var function.  It will be a ), but it may not be the first one.
            // for example: var(--secondary-text, rgba(0, 0, 0, 0.55))
            let openParens = 0;
            let indexOfEndOfVar = -1;
            for (let j = indexOfVar + varString.length; j < line.length; j++)
            {
                let check = line[j];

                // Keep track of opened parens.  If we hit a closing paren and there were no open parens,
                // we found the end of the var.
                if (check === '(') {
                    openParens++;
                }
                else if (check === ')') {
                    if (openParens === 0) {
                        indexOfEndOfVar = j;
                        break;
                    }
                    else {
                        openParens--;
                    }
                }
            }

            if (indexOfEndOfVar > -1) {
                // Store everything after the var function
                let postvar = line.substr(indexOfEndOfVar + 1).trim();
                let endOfLine = "";
                // If the text post var contains a }, we need to remove it.  It may contain more than 1.
                while (postvar.endsWith("}"))
                {
                    endOfLine += "}";
                    postvar = postvar.substr(0, postvar.length - 1).trim();
                }

                // Grab the inputs to the var function and store the theme variable that needs to be replaced.
                const varInput = line.substr(indexOfVar + 4, indexOfEndOfVar - indexOfVar - 4);
                const varInputs = varInput.split(",");
                let themeEntry = varInputs[0].trim();

                const replacementVar = defaults[themeEntry.substr(2)];
                if (replacementVar) {
                    const duplicateAttribute = `${attribute}:${prevar}${replacementVar}${postvar}`;
                    outputLines.push(duplicateAttribute);
                }
                else {
                    console.error(`Did not find replacement variable for: '${themeEntry}'. Line: ${line}`);
                }

                // If no default was set, then write the line back out with a default.
                if (varInputs.length !== 2) {

                    writeOriginalLine = false;

                    // Here are components of this output.
                    // If the line being processed is:
                    //   background-color: rgba( var(--palette-primary) ,  0.1 ); }
                    //
                    // {0}: attribute = background-color
                    // {1}: prevar =  rgba( 
                    // {2}: themeEntry = --palette-primary
                    // {3}: fallback value
                    // {4}: postvar - this does not include } at end of line =  ,  0.1 );
                    // {5}: endOfLine = }
                    //
                    // Output line will be:
                    //  background-color: rgba( var(--palette-primary,#0078d4),  0.1 );}
                    const updatedAttribute = `${attribute}:${prevar}var(${themeEntry},${replacementVar})${postvar}${endOfLine}`;
                    outputLines.push(updatedAttribute);
                }
            }
            else {
                console.error(`Did not find closing bracket for var function: ${line}`);
            }
        }
    }

    if (writeOriginalLine) {
        outputLines.push(line);
    }
}

function processCssContent(content, defaults) {
    const originalLines = content.split("\n");
    const outputLines = [];

    originalLines.forEach((line) => {
        processCssLine(line, defaults, outputLines);
    });

    return outputLines.join("\n");
}

function loadDefaultValues(defaultFilePaths) {

    let defaultsPaths = [ path.join(__dirname, "cssDefaults.json") ];

    if (defaultFilePaths) {
        defaultsPaths = defaultsPaths.concat(defaultFilePaths);
    }

    let defaults = {};
    defaultsPaths.forEach((file) => {
        defaults = { ...defaults, ...JSON.parse(fs.readFileSync(file, 'utf8')) };
    });

    return defaults;
}

module.exports = {
    processCssContent,
    loadDefaultValues
};